<%
    example.Text = "Example";
%>

<html>
    <head>
        <title>Sample ASP.Net Page</title>
    </head>
    <body>
        <form runat='server'>
            <asp:Label runat='server' id='example'></asp:Label>
        </form>
    </body>
</html>
